// Model object
//

#import <stdlib.h>
#import <math.h>
#import <string.h>
#import <ctype.h>

#import <activity.h>

#import "RDebug.h"
#import "RParameter.h"
#import "Model.h"
#import "World.h"
#import "Org.h"
#import "Pop.h"

extern id xZone;

@implementation Model

////////////////////////////////////////////////////
//
// First the real meat of the model actions.
// prestep, step, poststep, stepReport
//

-prestep {  // increment T, print a debug msg.
	++T;
	fprintf(reportFILE,"=====> Start Step %u.\n", T );
	return self;
}

-step {  // the model dynamics could be invoked here
	id		index, o;

	[world activateNewChoicesFor: org ReportOn: reportFILE];
	[world activateNewProblemsFor: org ReportOn: reportFILE];

	if ( RDebug > 2 ) {
		SOMsg1("\n-> START Model-step T %u  ========================\n",T);
		[org displayOrgOn: stdout];
	}

	[[org getopenProblems] forEach: M(tryAttachToOpenChoicesOfOrg:) : org];

	[[org getagents] forEach: M(tryAttachToOpenChoicesOfOrg:) : org];

	[[org getopenChoices] forEach: M(updateE)];

	if ( RDebug > 3 ) {
		SOMsg1( "==> after updating energy for step %u:\n", T );
		[org displayOrgOn: stdout];
		SOMsg( "==> now try to make choices...\n" );
	}

	// can I use forEach for this, including the removal??

	// [[org getopenChoices] forEach: M(tryMakeChoice)];
   	index = [[org getopenChoices] begin: xZone];
   	while ( (o = [index next]) ) {
		if ( [o tryMakeChoice] ) {
			[index remove];
		}
	} [index dropFrom: xZone];

	// now some counting...
	[[org getopenProblems] forEach: M(updateOpenStats)];
	[[org getagents] forEach: M(updateStats)];

	if ( RDebug > 1 ) {
		SOMsg1( "==> END of step %u:\n", T );
		[org displayOrgOn: stdout];
		[org printFinalReport: stdout];
		SOMsg( "============================================\n" );
	}

	return self;
}

// poststep

-poststep {

	return self;
}

-testHalt {	

	if ( T == stopT ) {	// generic stop condition
		[getTopLevelActivity() terminate];
	}
	if ( !stopOnlyAtT ) {
		// fill in specific stop condition, if needed
		// but only used if stopOnlyAtT is not turned on.
		;
	}
	return self;
}

///////////////////////////////////////////////////////
// stepReport

-stepReport {
   	fprintf(reportFILE, "=======> END of step %u:\n", T );
   	[org stepReport: reportFILE];
	return self;
}

//////////////////////////////////////////////////////////
// finish
// Sometimes it useful to re-print out final report step,
// or maybe print some run summary report.
// and do all the tidying up, eg closing files, etc.

-finish {

	// model specific processing here
	fprintf(reportFILE,"##########\n");
	//	fprintf(reportFILE,"#");  // rewrite the last report data line on # line
	// [self stepReport];
	fprintf(reportFILE,"##########\n");
	fprintf(reportFILE,"# Model run done at %s.\n", [self getTime] );

	[org printFinalReport: reportFILE];

	// All models need this...
	if ( fclose( reportFILE ) ) {
		fprintf(stderr,"\n\nError closing reportFILE (%s).\n\n",
				[self getStrParValueFor: "reportFileName"] );
	}

	return self;
}

//////////////////////////////////////////////////////////
// init

-init: (id) aZone ArgC: (int) argc ArgV: (char **) argv {
	id		p;
	int		i, j;

	// setup parameters specific to this model.
	// first add to list, setting defaults.
	ProgName = argv[0];
	Version = "1.0";
	[self setExpRunPar: "stopT" To: "20"];

	p = [RParameter createBegin: aZone];
	p = [p setName: "decstr" Aka: "ds" Type: charPar Value: "u"];
	p = [p createEnd: self];	[Parameters addLast: p];

	p = [RParameter createBegin: aZone];
	p = [p setName: "accstr" Aka: "as" Type: charPar Value: "u"];
	p = [p createEnd: self];	[Parameters addLast: p];

	p = [RParameter createBegin: aZone];
	p = [p setName: "choices" Aka: "" Type: stringPar Value: ""];
	p = [p createEnd: self];	[Parameters addLast: p];

	p = [RParameter createBegin: aZone];
	p = [p setName: "problems" Aka: "" Type: stringPar Value: ""];
	p = [p createEnd: self];	[Parameters addLast: p];

	p = [RParameter createBegin: aZone];
	p = [p setName: "agents" Aka: "" Type: stringPar Value: ""];
	p = [p createEnd: self];	[Parameters addLast: p];

	p = [RParameter createBegin: aZone];
	p = [p setName: "solncoeffs" Aka: "" Type: stringPar Value: ""];
	p = [p createEnd: self];	[Parameters addLast: p];

	p = [RParameter createBegin: aZone];
	p = [p setLB: 0.1];  p = [p setUB: 10.0];
	p = [p setName: "eload" Aka: "" Type: doublePar Value: "1.1"];
	p = [p createEnd: self];	[Parameters addLast: p];

	p = [RParameter createBegin: aZone];
	p = [p setName: "useii" Aka: "" Type: boolPar Value: "1"];
	p = [p createEnd: self];	[Parameters addLast: p];

	// Now read in the run time parameters, then init the RNG.
	[self processCmdLineParameters: aZone ArgC: argc ArgV: argv];
	[self startRNG];

	// Now create and initialize parts of this specific model

	world = [World createBegin: aZone];
	[world setmodel: self];
	world = [world createEnd: aZone];

	org = [Org createBegin: aZone];
	[org setmodel: self];
	org = [org createEnd: aZone];

	pop = [Pop createBegin: aZone];
	[pop setmodel: self];
	[pop setorg: org];			// this must be after org set up!
	[pop setworld: world];
	pop = [pop createEnd: aZone];

	[world setorg: org];
	[world setpop: pop];
	[org setworld: world];
	[org setpop: pop];
	[org setagents: [pop getagents]];
	[[org getagents] forEach: M(setorg:) : org];

	[world initProblemsAndChoices: aZone];

	// open report file, write header info, and we're ready to go!
	[self startReportFile];

	// for initializing special arrays for comparing to fortran output
	for ( i = 0; i < GCanHistorySz; ++i ) {
		for ( j = 0; j < GCanHistorySz; ++j ) {
			ChoiceHistory[i][j] = 0;
			DMHistory[i][j] = 0;
			ProbHistory[i][j] = -1;
		}
	}

	return self;
}


//////////////////////////////////////////////////////////////////
// startReportFile
// After generic message sent to start the report file
// (opens file and writes Parameters list values),
// add any model-specific lines (eg column headers)
// ==> Be sure to start non-data lines with #'s.

-startReportFile {

	[self startExpRunReportFile];

	fprintf(reportFILE,"#\n" );	// start model specifics here

	[org printstepReportFormat: reportFILE];

	fflush(reportFILE);

	return self;
}

////////////////////////////////////////////////////////////
// get/set - stopOnlyAtT
// for faster access to this variable which is
// also on the parameter list

-(BOOL) getstopOnlyAtT { return stopOnlyAtT; }
-setstopOnlyAtT: (BOOL) b {
	// ToDo => set in parameter list
	stopOnlyAtT = b;
	return self;
}

-getorg {
	return org;
}

@end








