#!/usr/local/bin/tclsh

set scriptName [file tail $argv0]
set DateTime [exec date +%C%m%d\-%H%M]

set verbose 0
set table1 0
set table2 0
set base   "x001"
set eLoad { 1.1 2.2 3.3 }

proc Usage args {
	global scriptName
	puts "Usage:  $scriptName parameters"
	puts "where parameters may be:"
    puts " -t1  produce table 1 from paper"
    puts " -t2  produce table 2 from paper"
	puts " -b<base>  base directory"
	puts " -v   verbose"
}

##########################################################################
# Script documentation starts here.
#
##########################################################################

# first  process run time parameters for this batch script

for {set rtp  0} {$rtp < $argc} {incr rtp} {
	if { [string match "-t1" [lindex $argv $rtp]] } {
		set table1 1
	} elseif { [string match "-t2" [lindex $argv $rtp]] } {
		set table2 1
	} elseif { [string match "-v" [lindex $argv $rtp]] } {
		set verbose 1
	} elseif { [string match "-b*" [lindex $argv $rtp]] } {
		scan [lindex $argv $rtp] "-b%s" base

	} else {
		puts "Unknown parameter '[lindex $argv $rtp]'."
		Usage
		puts "Bye."
		exit
	}
}

####################################

proc Table1 args {
global base verbose

set eLoad { 1.1 2.2 3.3 }
set eDistr { U H S }
set decStr { U H S }
set accStr { U H S }

set cntAll       0
set fracAllRB    0

foreach AS $accStr {
	foreach EL $eLoad {
		set Fraction($AS-$EL) 0
		set Cnt($AS-$EL) 0
		set ChoicesMade($AS-$EL) 0
	}
}

# End of report output we are looking for looks like:
#F  Org final report:
#F  R=Resolution    O=Oversite     F=Flight        B=Both R,F
#F  KZ=chOpenCnt    KY=chDiffCnt   XS=chWastedE
#F  KX=agActivCnt   KS=agRestCnt   KR=agUnsedE
#F  KW=prOpenCnt    KV=prActivCnt  KU=prLatentCnt  KT=prAttachCnt
#F     R    O    F    B   KZ  KY    XS   KX  KS    KR   KW  KV  KU  KT
#FRN 0.50 0.50 0.00 0.00   0   4  11.00  30   0   0.00   6   2  16   0
####

set allKZ 0
set allKW 0

foreach AS $accStr {
	foreach EL $eLoad {
		set pattern "EL=$EL-ED=*-DS=*-AC=$AS-case*.in-pars.report"
		set files [glob "$base/$pattern"]
		if { $verbose } { puts "pattern($AS-$EL): $base/$pattern" }
		set cntFnd 0
		foreach file $files {
			set FILE [open $file r]
			while { [gets $FILE line] >= 0 } {
				if { [regexp {^#FRN ([^\ ]*) ([^\ ]*) ([^\ ]*) ([^\ ]*)} $line match R O F B] } {
					incr cntFnd
					incr cntAll
					set fracAllRB [expr $fracAllRB + $R + $B]
					set Fraction($AS-$EL) [expr $Fraction($AS-$EL) + $R + $B]
					set Cnt($AS-$EL) [expr $Cnt($AS-$EL) + 1]

					set line [string range $line [string length $match] 9999]
					regexp {\ *([^\ ]*)\ *([^\ ]*)\ *([^\ ]*)\ *([^\ ]*)} $line match KZ KY XS KX

					set line [string range $line [string length $match] 9999]
					regexp {\ *([^\ ]*)\ *([^\ ]*)\ *([^\ ]*)\ *([^\ ]*)} $line match KS KR KW KV

					set line [string range $line [string length $match] 9999]
					regexp {\ *([^\ ]*)\ *([^\ ]*)} $line match KU KT

					set allKZ [expr $allKZ + $KZ]
					set allKW [expr $allKW + $KW]

					set ChoicesMade($AS-$EL) [expr $ChoicesMade($AS-$EL) + (10 - $KZ)]

					break
				}
			}
			close $FILE
		}
		#puts "For ($AS-$EL) case, $Cnt($AS-$EL) files: Found $cntFnd FRN lines."
		#set f [expr $Fraction($AS-$EL) / $Cnt($AS-$EL)]
		#puts [format "   -- Fraction($AS-$EL)=%.2f fraction %4.2f" $Fraction($AS-$EL) $f ]
		#puts [format "   -- fracAllRB %.2f, cntAll $cntAll." $fracAllRB]
	}
}

foreach EL $eLoad {
	foreach AS $accStr {
		set FractionMade($AS-$EL) [expr $Fraction($AS-$EL) / $Cnt($AS-$EL)]
	}
}

puts ""
puts "Table 1  Fraction of Choices Made that Resolve Problems"
puts ""
puts "               Access Structure"
puts "ELoad       All         U         H         S"
foreach EL $eLoad {
	puts -nonewline " $EL    "
	set FractionMade($EL) 0
	set Cnt($EL) 0
	foreach AS $accStr {
		set FractionMade($EL) [expr $FractionMade($EL) + $FractionMade($AS-$EL)]
		set Cnt($EL) [expr $Cnt($EL) + 1]
	}
	set FractionMade($EL) [expr $FractionMade($EL) / $Cnt($EL)]
	puts -nonewline [format "   %4.2f " $FractionMade($EL)] 
	foreach AS $accStr {
		puts -nonewline [format "      %4.2f" $FractionMade($AS-$EL)] 
	}
	puts ""
}

set f [expr $fracAllRB / $cntAll]
puts -nonewline [format " All       %4.2f  " $f]

foreach AS $accStr {
	set FractionMade($AS) 0
	set Cnt($AS) 0
	foreach EL $eLoad {
		set FractionMade($AS) [expr $FractionMade($AS) + $FractionMade($AS-$EL)]
		set Cnt($AS) [expr $Cnt($AS) + 1]
	}
	set FractionMade($AS) [expr $FractionMade($AS) / $Cnt($AS)]
	puts -nonewline [format "     %4.2f " $FractionMade($AS)]
} 
puts ""

puts ""

puts "allKZ $allKZ, allKW $allKW,  cntAll $cntAll"

set allKZ [expr 1.0 * $allKZ / $cntAll]
set allKW [expr 1.0 * $allKW / $cntAll]

puts [format "OpenChoices  (KZ): %6.2f" $allKZ]
puts [format "OpenProblems (KW): %6.2f" $allKW]
puts "--------------------------------------------"

#puts [format "fracAllRB %.2f, $cntAll" $fracAllRB]
#puts ""

foreach EL $eLoad {
	foreach AS $accStr {
		# to get to Chresolved as propor of *all* choices, get *number* that resolved div 10
		set FractionAll($AS-$EL) [expr $FractionMade($AS-$EL) * $ChoicesMade($AS-$EL) / 10]
		set FractionAll($AS-$EL) [expr $FractionAll($AS-$EL) / $Cnt($AS-$EL)]
	}
}

puts ""
puts "Table 1  Fraction of All Choices that Resolve Problems"
puts ""
puts "               Access Structure"
puts "ELoad       All         U         H         S"
foreach EL $eLoad {
	puts -nonewline " $EL    "
	set FractionAll($EL) 0
	set Cnt($EL) 0
	foreach AS $accStr {
		set FractionAll($EL) [expr $FractionAll($EL) + $FractionAll($AS-$EL)]
		set Cnt($EL) [expr $Cnt($EL) + 1]
	}
	set FractionAll($EL) [expr $FractionAll($EL) / $Cnt($EL)]
	puts -nonewline [format "   %4.2f " $FractionAll($EL)] 
	foreach AS $accStr {
		puts -nonewline [format "      %4.2f" $FractionAll($AS-$EL)] 
	}
	puts ""
}

puts -nonewline "      "

foreach AS $accStr {
	set FractionAll($AS) 0
	set Cnt($AS) 0
	foreach EL $eLoad {
		set FractionAll($AS) [expr $FractionAll($AS) + $FractionAll($AS-$EL)]
		set Cnt($AS) [expr $Cnt($AS) + 1]
	}
	set FractionAll($AS) [expr $FractionAll($AS) / $Cnt($AS)]
	puts -nonewline [format "     %4.2f " $FractionAll($AS)]
} 
puts ""

puts "--------------------------------------------"
}
# end of proc Table1


##########################################################################

proc Table2 args {
global base verbose

set eLoad { 1.1 2.2 3.3 }
set eDistr { U H S }
set decStr { U H S }
set accStr { U H S }

set cntAll       0
set fracAllFO    0

# Fraction  - of F O at the ELoad
# probActiv - KT
# dmActiv   - KX
# decDiff   - KY

foreach EL $eLoad {
	set Fraction($EL) 0
	set probActiv($EL) 0
	set dmActiv($EL) 0
	set decDiff($EL) 0
	set Cnt($EL) 0
}

# End of report output we are looking for looks like:
#F  Org final report:
#F  R=Resolution    O=Oversite     F=Flight        B=Both R,F
#F  KZ=chOpenCnt    KY=chDiffCnt   XS=chWastedE
#F  KX=agActivCnt   KS=agRestCnt   KR=agUnsedE
#F  KW=prOpenCnt    KV=prActivCnt  KU=prLatentCnt  KT=prAttachCnt
#F     R    O    F    B   KZ  KY    XS   KX  KS    KR   KW  KV  KU  KT
#FRN 0.50 0.50 0.00 0.00   0   4  11.00  30   0   0.00   6   2  16   0
####

	foreach EL $eLoad {
		set pattern "EL=$EL-ED=*-DS=*-AC=*-case*.in-pars.report"
		set files [glob "$base/$pattern"]
		if { $verbose } { puts "pattern($EL): $base/$pattern" }
		set cntFnd 0
		foreach file $files {
			set FILE [open $file r]
			while { [gets $FILE line] >= 0 } {
				if { [regexp {^#FRN ([^\ ]*) ([^\ ]*) ([^\ ]*) ([^\ ]*)} $line match R O F B] } {
					incr cntFnd
					incr cntAll
					set fracAllFO [expr $fracAllFO + $F + $O]
					set Fraction($EL) [expr $Fraction($EL) + $F + $O]
					set Cnt($EL) [expr $Cnt($EL) + 1]

					set line [string range $line [string length $match] 9999]
					regexp {\ *([^\ ]*)\ *([^\ ]*)\ *([^\ ]*)\ *([^\ ]*)} $line match KZ KY XS KX

					set line [string range $line [string length $match] 9999]
					regexp {\ *([^\ ]*)\ *([^\ ]*)\ *([^\ ]*)\ *([^\ ]*)} $line match KS KR KW KV

					set line [string range $line [string length $match] 9999]
					regexp {\ *([^\ ]*)\ *([^\ ]*)} $line match KU KT

					set probActiv($EL) [expr $probActiv($EL) + $KT]
					set dmActiv($EL)   [expr $dmActiv($EL) + $KX]
					set decDiff($EL) [expr $decDiff($EL) + $KY]

					break
				}
			}
			close $FILE
		}
	}


	foreach EL $eLoad {
		set Fraction($EL) [expr $Fraction($EL) / $Cnt($EL)]
		set probActiv($EL) [expr $probActiv($EL) / $Cnt($EL)]
		set dmActiv($EL)   [expr $dmActiv($EL) / $Cnt($EL)]
		set decDiff($EL) [expr $decDiff($EL) / $Cnt($EL)]
	}

	puts ""
	puts "Table 2  Effects of Load Variation"
	puts "       Problem   DecMaker   Decision     Choices by"
	puts "ELoad  Activity  Activity  Difficulty  Flight/Oversite"
	foreach EL $eLoad {
		puts -nonewline [format " $EL    %6.1f   %6.1f     %6.1f       %5.2f" \
				$probActiv($EL) $dmActiv($EL) $decDiff($EL) $Fraction($EL)]
		puts ""
	}

	puts "------------------------------------------"
}
# end of proc Table2
#####################################################################################

if { $table1 } {
	Table1
}
if { $table2 } {
	Table2
}

########################################################################################
# Table 3: fraction of problems resolved, for AS=H, by:
#    Importance of problem: first-10, last-10
#    Arrival of problem:    first-10, last-10
# The format of a report line with for entering and solved problems:
# PE: entryT Id Import
# PS: entryT solveT Id Import choiceMethod  ChoiceId
# Recall lower N means higher importance

set pattern "EL=*-ED=*-DS=*-AC=H-case*.in-pars.report"
set files [glob "$base/$pattern"]
puts ""
if { $verbose } { puts "Table3 pattern: $base/$pattern" }
set fcnt [llength $files]
set cntResFnd 0
set cntEnterFnd 0
set earlyLowICnt 0
set earlyHiICnt  0
set lateLowICnt  0
set lateHiICnt   0
set earlyLowIRes 0
set earlyHiIRes  0
set lateLowIRes  0
set lateHiIRes   0

foreach file $files {
	set FILE [open $file r]
	while { [gets $FILE line] >= 0 } {
		if { [regexp {^PS:\ *([^\ ]*)\ *([^\ ]*)\ *([^\ ]*)\ *([^\ ]*)} $line match ET ST Id Import] } {
			incr cntResFnd
			if { $ET <= 5 && $Import <= 10 } {
				incr earlyHiIRes
			} elseif { $ET <= 5 && $Import > 10 } {
				incr earlyLowIRes
			} elseif { $ET > 5 && $Import <= 10 } {
				incr lateHiIRes
			} elseif { $ET > 5 && $Import > 10 } {
				incr lateLowIRes
			}
		} elseif { [regexp {^PE:\ *([^\ ]*)\ *([^\ ]*)\ *([^\ ]*)} $line match ET Id Import] } {
			incr cntEnterFnd
			if { $ET <= 5 && $Import <= 10 } {
				incr earlyHiICnt
			} elseif { $ET <= 5 && $Import > 10 } {
				incr earlyLowICnt
			} elseif { $ET > 5 && $Import <= 10 } {
				incr lateHiICnt
			} elseif { $ET > 5 && $Import > 10 } {
				incr lateLowICnt
			}
		}
	}
	close $FILE
}

puts "Table 3  Fraction Problems Resolved (incl. Flight&Resolve)"
puts "         (files $fcnt, PS EnterCount $cntEnterFnd, ResCnt $cntResFnd)"

puts "early-Low/High-EntryCnt $earlyLowICnt, $earlyHiICnt"
puts "late-Low/High-EntryCnt  $lateLowICnt, $lateHiICnt"
puts "early-Low/HighIRes $earlyLowIRes, $earlyHiIRes"
puts "late-Low/HighIRes  $lateLowIRes, $lateHiIRes"

set EHi  [expr 1.0 * $earlyHiIRes / $earlyHiICnt]
set LHi  [expr 1.0 * $lateHiIRes / $lateHiICnt]
set ELo  [expr 1.0 * $earlyLowIRes / $earlyLowICnt]
set LLo  [expr 1.0 * $lateLowIRes / $lateLowICnt]
puts ""
puts "               Early     Late"
puts "               -----     ----"
puts [format "High Import   %6.2f   %6.2f" $EHi $LHi] 
puts [format "Low Import    %6.2f   %6.2f" $ELo $LLo] 
puts ""
puts "-----------------------------------------------"

##############################################################
# Table 4
# Fraction choices made by F or O (AS=H *OR* DS=H), by
#   Importance of Choice:    top-5,  bottom-5
#   Arrival of Choice:     first-5,    last-5
# Format of report lines for entering and made choices:
# CE: entryT Id Import
# CM: entryT decT Id Import choiceMethod ( AgId, Rank, E ) ...
#

set pattern "EL=*-ED=*-DS=*-AC=H-case*.in-pars.report"
set files [glob "$base/$pattern"]
set pattern "EL=*-ED=*-DS=H-AC=S-case*.in-pars.report"
set files1 [glob "$base/$pattern"]
set pattern "EL=*-ED=*-DS=H-AC=U-case*.in-pars.report"
set files2 [glob "$base/$pattern"]
set files [concat $files $files1 $files2]
set fcnt [llength $files]

set cntFnd 0
set cntEnterFnd 0
set earlyLowICnt 0
set earlyHiICnt  0
set lateLowICnt  0
set lateHiICnt   0
set earlyLowIFO 0
set earlyHiIFO  0
set lateLowIFO  0
set lateHiIFO   0

foreach file $files {
	set FILE [open $file r]
	while { [gets $FILE line] >= 0 } {
		if { [regexp {^CM:\ *([^\ ]*)\ *([^\ ]*)\ *([^\ ]*)\ *([^\ ]*)\ *([^\ ]*)} $line match ET DT Id Import CM] } {
			incr cntFnd
			if { $Import > 10 } {
				puts "ET $ET  ST $ST Import $Import"
			}
			if { $ET <=5 && $Import <= 5 } {
				if { $CM == "F" || $CM == "O" } { incr earlyHiIFO }
			} elseif { $ET <= 5 && $Import > 5 } {
				if { $CM == "F" || $CM == "O" } { incr earlyLowIFO }
			} elseif { $ET > 5 && $Import <= 5 } {
				if { $CM == "F" || $CM == "O" } { incr lateHiIFO }
			} elseif { $ET > 5 && $Import > 5 } {
				if { $CM == "F" || $CM == "O" } { incr lateLowIFO }
			}
		} elseif { [regexp {^CE:\ *([^\ ]*)\ *([^\ ]*)\ *([^\ ]*)} $line match ET Id Import] } {
			incr cntEnterFnd
			if { $ET <= 5 && $Import <= 5 } {
				incr earlyHiICnt
			} elseif { $ET <= 5 && $Import > 5 } {
				incr earlyLowICnt
			} elseif { $ET > 5 && $Import <= 5 } {
				incr lateHiICnt
			} elseif { $ET > 5 && $Import > 5 } {
				incr lateLowICnt
			}
		}
	}
	close $FILE
}

puts ""
puts "Table 4  Fraction Choices Made by Flight or Oversight"  
puts "         (files $fcnt;  ChoiceEntry $cntEnterFnd, ChoiceMade $cntFnd)"

if { $verbose } {
	puts "early-Low/High-EntryCnt $earlyLowICnt, $earlyHiICnt"
	puts "late-Low/High-EntryCnt  $lateLowICnt, $lateHiICnt"
	puts "early-Low/HighIRes $earlyLowIFO, $earlyHiIFO"
	puts "late-Low/HighIRes  $lateLowIFO, $lateHiIFO"
}

set EHi 0
set LHi 0
set ELo 0
set LLo 0
if { $earlyHiICnt > 0 } { set EHi  [expr 1.0 * $earlyHiIFO / $earlyHiICnt] }
if { $lateHiICnt > 0 } { set LHi  [expr 1.0 * $lateHiIFO / $lateHiICnt] }
if { $earlyLowICnt > 0 } { set ELo  [expr 1.0 * $earlyLowIFO / $earlyLowICnt] }
if { $lateLowICnt > 0 } { set LLo  [expr 1.0 * $lateLowIFO / $lateLowICnt] }

puts ""
puts "               Early     Late"
puts "               -----     ----"
puts [format "High Import   %6.2f   %6.2f" $EHi $LHi] 
puts [format "Low Import    %6.2f   %6.2f" $ELo $LLo] 
puts ""
puts "-----------------------------------------------------"

########################################################################
#
# Prop. Choices not made (p11, under table 4)
# Hier Acc and Dec structures
# By Choice Importance (first 3, middle 4, least 3)
#


