// Copyright (C) 1995 The Santa Fe Institute.
// No warranty implied, see LICENSE for terms.

// test for ExpRun and Parameter objects.

#import <math.h>
#import <stdlib.h>

#import <collections.h>
#import <tkobjc.h>
#import <space.h>
#import <simtools.h>
#import <activity.h>
#import <random.h>

#define main_m		// for RDebug.h
#import "RDebug.h"
#import	"Model.h"

id xZone;	// global scrap area (for debugging)

int		DisplayFrequency = 1;		//  What to do with this???

int main(int argc, char **argv) {
	id  mZone;
	id  simActions, displayActions;		  // list of actions for sim
  	id 	simSchedule, displaySchedule;		  // schedules
  	id 	swarmPlan;					  // swarm plan for simulation
  	id 	swarmActivity;				  // executor of swarm plan
	id  model;

	initBatchSwarm(argc,argv);
	mZone = [Zone create: NULL];
	xZone = [Zone create: NULL];

	model = [Model createBegin: mZone];
	model = [model createEnd: mZone];
	[model init: mZone ArgC: argc ArgV: argv];

  	// First create the list of simulation actions. We put these in an
  	// action group, because we want these actions to be executed in a
  	// specific order, but these steps should take no (simulated)
  	// time. The $m(foo) means "The message called <foo>". You can send
  	// a message To a particular object, or ForEach object in a collection.
  	simActions = [ActionGroup create: mZone];  
	[simActions createActionTo: model message: M(prestep)];
	[simActions createActionTo: model message: M(step)];
	[simActions createActionTo: model message: M(poststep)];

	simSchedule = [Schedule createBegin: mZone];
  	[simSchedule setRepeatInterval: 1];
  	simSchedule = [simSchedule createEnd];
	[simSchedule at: 0 createActionTo: simActions message: M(perform)];

  	// Create a group of display actions and a schedule for them.
	// ==> NOTE: also test for halting here, *after* the stepReport
  	displaySchedule = [Schedule createBegin: mZone];
  	[displaySchedule setRepeatInterval: DisplayFrequency];
  	displaySchedule = [displaySchedule createEnd];
	[displaySchedule at: 0 createActionTo: model message: M(stepReport)];
	[displaySchedule at: 0 createActionTo: model message: M(testHalt)];

  	// SwarmPlan to run the schedules
  	swarmPlan = [SwarmPlan create: mZone];
  	[swarmPlan createActionTo: simSchedule message: M(start)];
  	[swarmPlan createActionTo: displaySchedule message: M(start)];

  	// A swarmActivity whose job it is to execute the swarmPlan
  	swarmActivity = [swarmPlan start];
   	[swarmActivity run];
	[model finish];

  	return 0;
}





